@@ -7,17 +7,17 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
+ * USA
  */
 
 #include "ibusimpl.h"
@@ -826,7 +826,8 @@ _context_focus_out_cb (BusInputContext    *context,
 /**
  * _context_destroy_cb:
  *
- * A callback function to be called when the "destroy" signal is sent to the context.
+ * A callback function to be called when the "destroy" signal is sent to the
+ * context.
  */
 static void
 _context_destroy_cb (BusInputContext    *context,
@@ -835,9 +836,12 @@ _context_destroy_cb (BusInputContext    *context,
     g_assert (BUS_IS_IBUS_IMPL (ibus));
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
-    if (context == ibus->focused_context) {
+    if (context == ibus->focused_context)
         bus_ibus_impl_set_focused_context (ibus, NULL);
-    }
+
+    if (ibus->panel &&
+        bus_input_context_get_capabilities (context) & IBUS_CAP_FOCUS)
+        bus_panel_proxy_destroy_context (ibus->panel, context);
 
     ibus->contexts = g_list_remove (ibus->contexts, context);
     g_object_unref (context);
