@@ -1,29 +1,32 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2010 Red Hat, Inc.
+ * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2013 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
+ * USA
  */
 
 #if !defined (__IBUS_H_INSIDE__) && !defined (IBUS_COMPILATION)
 #error "Only <ibus.h> can be included directly"
 #endif
 
+#ifndef __IBUS_CONFIG_SERVICE_H_
+#define __IBUS_CONFIG_SERVICE_H_
+
 /**
  * SECTION: ibusconfigservice
  * @short_description: Configuration service back-end.
@@ -95,8 +98,8 @@
  *             <varlistentry>
  *                 <term>Returns:</term>
  *                 <listitem>TRUE if succeed; FALSE otherwise.</listitem>
-*             </varlistentry>
-*         </variablelist>
+ *             </varlistentry>
+ *        </variablelist>
  *        <para>Get value of a configuration option.
  *        </para>
  *     </listitem>
@@ -131,8 +134,6 @@
  *     </listitem>
  * </itemizedlist>
  */
-#ifndef __IBUS_CONFIG_SERVICE_H_
-#define __IBUS_CONFIG_SERVICE_H_
 
 #include "ibusservice.h"
 
@@ -188,7 +189,6 @@ struct _IBusConfigServiceClass {
      * @config: An IBusConfig.
      * @section: section name
      * @name: value name
-     *
      * @returns: (transfer full): The value in config associated with section and name.
      *
      */
@@ -200,10 +200,13 @@ struct _IBusConfigServiceClass {
                                  const gchar          *section,
                                  const gchar          *name,
                                  GError              **error);
+    GVariant *  (* get_values)  (IBusConfigService    *config,
+                                 const gchar          *section,
+                                 GError              **error);
 
     /*< private >*/
     /* padding */
-    gpointer pdummy[13];
+    gpointer pdummy[12];
 };
 
 GType                ibus_config_service_get_type   (void);
