@@ -230,7 +230,9 @@ ibus_error_get_type (void)
     static GType etype = 0;
     if (G_UNLIKELY(etype == 0)) {
         static const GEnumValue values[] = {
-            { IBUS_ERROR_NO_ENGINE, "IBUS_ERROR_NO_ENGINE", "engine" },
+            { IBUS_ERROR_NO_ENGINE, "IBUS_ERROR_NO_ENGINE", "no-engine" },
+            { IBUS_ERROR_NO_CONFIG, "IBUS_ERROR_NO_CONFIG", "no-config" },
+            { IBUS_ERROR_FAILED, "IBUS_ERROR_FAILED", "failed" },
             { 0, NULL, NULL }
         };
         etype = g_enum_register_static (g_intern_static_string ("IBusError"), values);
@@ -238,6 +240,51 @@ ibus_error_get_type (void)
     return etype;
 }
 
+GType
+ibus_input_purpose_get_type (void)
+{
+    static GType etype = 0;
+    if (G_UNLIKELY(etype == 0)) {
+        static const GEnumValue values[] = {
+            { IBUS_INPUT_PURPOSE_FREE_FORM, "IBUS_INPUT_PURPOSE_FREE_FORM", "free-form" },
+            { IBUS_INPUT_PURPOSE_ALPHA, "IBUS_INPUT_PURPOSE_ALPHA", "alpha" },
+            { IBUS_INPUT_PURPOSE_DIGITS, "IBUS_INPUT_PURPOSE_DIGITS", "digits" },
+            { IBUS_INPUT_PURPOSE_NUMBER, "IBUS_INPUT_PURPOSE_NUMBER", "number" },
+            { IBUS_INPUT_PURPOSE_PHONE, "IBUS_INPUT_PURPOSE_PHONE", "phone" },
+            { IBUS_INPUT_PURPOSE_URL, "IBUS_INPUT_PURPOSE_URL", "url" },
+            { IBUS_INPUT_PURPOSE_EMAIL, "IBUS_INPUT_PURPOSE_EMAIL", "email" },
+            { IBUS_INPUT_PURPOSE_NAME, "IBUS_INPUT_PURPOSE_NAME", "name" },
+            { IBUS_INPUT_PURPOSE_PASSWORD, "IBUS_INPUT_PURPOSE_PASSWORD", "password" },
+            { IBUS_INPUT_PURPOSE_PIN, "IBUS_INPUT_PURPOSE_PIN", "pin" },
+            { 0, NULL, NULL }
+        };
+        etype = g_enum_register_static (g_intern_static_string ("IBusInputPurpose"), values);
+    }
+    return etype;
+}
+
+GType
+ibus_input_hints_get_type (void)
+{
+    static GType etype = 0;
+    if (G_UNLIKELY(etype == 0)) {
+        static const GFlagsValue values[] = {
+            { IBUS_INPUT_HINT_NONE, "IBUS_INPUT_HINT_NONE", "none" },
+            { IBUS_INPUT_HINT_SPELLCHECK, "IBUS_INPUT_HINT_SPELLCHECK", "spellcheck" },
+            { IBUS_INPUT_HINT_NO_SPELLCHECK, "IBUS_INPUT_HINT_NO_SPELLCHECK", "no-spellcheck" },
+            { IBUS_INPUT_HINT_WORD_COMPLETION, "IBUS_INPUT_HINT_WORD_COMPLETION", "word-completion" },
+            { IBUS_INPUT_HINT_LOWERCASE, "IBUS_INPUT_HINT_LOWERCASE", "lowercase" },
+            { IBUS_INPUT_HINT_UPPERCASE_CHARS, "IBUS_INPUT_HINT_UPPERCASE_CHARS", "uppercase-chars" },
+            { IBUS_INPUT_HINT_UPPERCASE_WORDS, "IBUS_INPUT_HINT_UPPERCASE_WORDS", "uppercase-words" },
+            { IBUS_INPUT_HINT_UPPERCASE_SENTENCES, "IBUS_INPUT_HINT_UPPERCASE_SENTENCES", "uppercase-sentences" },
+            { IBUS_INPUT_HINT_INHIBIT_OSK, "IBUS_INPUT_HINT_INHIBIT_OSK", "inhibit-osk" },
+            { 0, NULL, NULL }
+        };
+        etype = g_flags_register_static (g_intern_static_string ("IBusInputHints"), values);
+    }
+    return etype;
+}
+
 
 /* Generated data ends here */
 
