@@ -1,23 +1,23 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2010 Red Hat, Inc.
+ * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2013 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
+ * USA
  */
 #include "ibusinputcontext.h"
 #include <gio/gio.h>
@@ -250,6 +250,8 @@ ibus_input_context_class_init (IBusInputContextClass *class)
     /**
      * IBusInputContext::update-auxiliary-text:
      * @context: An IBusInputContext.
+     * @text: An auxiliary text
+     * @visible: The visibility of @text
      *
      * Emitted to hide auxilary text.
      *
@@ -685,8 +687,7 @@ ibus_input_context_new (const gchar     *path,
 
     GInitable *initable;
 
-    GDBusProxyFlags flags = G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START |
-                            G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES;
+    GDBusProxyFlags flags = G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START;
 
     initable = g_initable_new (IBUS_TYPE_INPUT_CONTEXT,
                                cancellable,
@@ -714,8 +715,7 @@ ibus_input_context_new_async (const gchar         *path,
     g_assert (G_IS_DBUS_CONNECTION (connection));
     g_assert (callback != NULL);
 
-    GDBusProxyFlags flags = G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START |
-                            G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES;
+    GDBusProxyFlags flags = G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START;
 
     g_async_initable_new_async (IBUS_TYPE_INPUT_CONTEXT,
                                 G_PRIORITY_DEFAULT,
@@ -1009,45 +1009,6 @@ ibus_input_context_property_hide (IBusInputContext *context,
                        );
 }
 
-void
-ibus_input_context_is_enabled_async (IBusInputContext   *context,
-                                     gint                timeout_msec,
-                                     GCancellable       *cancellable,
-                                     GAsyncReadyCallback callback,
-                                     gpointer            user_data)
-{
-    g_assert (IBUS_IS_INPUT_CONTEXT (context));
-    g_dbus_proxy_call ((GDBusProxy *) context,
-                       "IsEnabled",               /* method_name */
-                       NULL,                      /* parameters */
-                       G_DBUS_CALL_FLAGS_NONE,    /* flags */
-                       timeout_msec,
-                       cancellable,
-                       callback,
-                       user_data);
-}
-
-gboolean
-ibus_input_context_is_enabled_async_finish (IBusInputContext   *context,
-                                            GAsyncResult       *res,
-                                            GError            **error)
-{
-    g_assert (IBUS_IS_INPUT_CONTEXT (context));
-    g_assert (G_IS_ASYNC_RESULT (res));
-    g_assert (error == NULL || *error == NULL);
-
-    gboolean enabled = FALSE;
-
-    GVariant *variant = g_dbus_proxy_call_finish ((GDBusProxy *) context,
-                                                   res, error);
-    if (variant != NULL) {
-        g_variant_get (variant, "(b)", &enabled);
-        g_variant_unref (variant);
-    }
-
-    return enabled;
-}
-
 void
 ibus_input_context_set_surrounding_text (IBusInputContext   *context,
                                          IBusText           *text,
@@ -1096,32 +1057,38 @@ ibus_input_context_needs_surrounding_text (IBusInputContext *context)
     return priv->needs_surrounding_text;
 }
 
-gboolean
-ibus_input_context_is_enabled (IBusInputContext *context)
+void
+ibus_input_context_set_content_type (IBusInputContext *context,
+                                     guint             purpose,
+                                     guint             hints)
 {
     g_assert (IBUS_IS_INPUT_CONTEXT (context));
-    GVariant *result;
-    GError *error = NULL;
-    result = g_dbus_proxy_call_sync ((GDBusProxy *) context,
-                                     "IsEnabled",               /* method_name */
-                                     NULL,                      /* parameters */
-                                     G_DBUS_CALL_FLAGS_NONE,    /* flags */
-                                     -1,                        /* timeout */
-                                     NULL,                      /* cancellable */
-                                     &error                     /* error */
-                                     );
 
-    if (result == NULL) {
-        g_warning ("%s.IsEnabled: %s", IBUS_INTERFACE_INPUT_CONTEXT, error->message);
-        g_error_free (error);
-        return FALSE;
+    GVariant *cached_content_type =
+        g_dbus_proxy_get_cached_property ((GDBusProxy *) context,
+                                          "ContentType");
+    GVariant *content_type = g_variant_new ("(uu)", purpose, hints);
+
+    g_variant_ref_sink (content_type);
+    if (cached_content_type == NULL ||
+        !g_variant_equal (content_type, cached_content_type)) {
+        g_dbus_proxy_call ((GDBusProxy *) context,
+                           "org.freedesktop.DBus.Properties.Set",
+                           g_variant_new ("(ssv)",
+                                          IBUS_INTERFACE_INPUT_CONTEXT,
+                                          "ContentType",
+                                          content_type),
+                           G_DBUS_CALL_FLAGS_NONE,
+                           -1,
+                           NULL, /* cancellable */
+                           NULL, /* callback */
+                           NULL  /* user_data */
+                           );
     }
 
-    gboolean retval = FALSE;
-    g_variant_get (result, "(b)", &retval);
-    g_variant_unref (result);
-
-    return retval;
+    if (cached_content_type != NULL)
+        g_variant_unref (cached_content_type);
+    g_variant_unref (content_type);
 }
 
 void
@@ -1242,6 +1209,4 @@ DEFINE_FUNC(page_up, PageUp);
 DEFINE_FUNC(page_down, PageDown);
 DEFINE_FUNC(cursor_up, CursorUp);
 DEFINE_FUNC(cursor_down, CursorDown);
-DEFINE_FUNC(enable, Enable);
-DEFINE_FUNC(disable, Disable);
 #undef DEFINE_FUNC
