@@ -1,29 +1,32 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input IBus
- * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2010 Red Hat, Inc.
+ * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2013 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
+ * USA
  */
 
 #if !defined (__IBUS_H_INSIDE__) && !defined (IBUS_COMPILATION)
 #error "Only <ibus.h> can be included directly"
 #endif
 
+#ifndef __IBUS_OBSERVED_PATH_H_
+#define __IBUS_OBSERVED_PATH_H_
+
 /**
  * SECTION: ibusobservedpath
  * @short_description: Path object of IBus.
@@ -33,9 +36,6 @@
  * such as monitor modification, directory tree traversal.
  */
 
-#ifndef __IBUS_OBSERVED_PATH_H_
-#define __IBUS_OBSERVED_PATH_H_
-
 #include "ibusserializable.h"
 #include "ibusxml.h"
 
@@ -116,12 +116,16 @@ IBusObservedPath    *ibus_observed_path_new                 (const gchar
 /**
  * ibus_observed_path_traverse:
  * @path: An IBusObservedPath.
- * @returns: (element-type IBusObservedPath): A newly allocate GList which holds content in path; NULL if @path is not directory.
+ * @dir_only: Only looks for subdirs, not files
+ * @returns: (transfer full) (element-type IBusObservedPath): A newly allocate
+ * GList which holds content in path; NULL if @path is not directory.
  *
- * Recursively traverse the path and put the files and subdirectory in to a newly allocated
+ * Recursively traverse the path and put the files and subdirectory in to
+ * a newly allocated
  * GLists, if the @path is a directory. Otherwise returns NULL.
  */
-GList               *ibus_observed_path_traverse            (IBusObservedPath   *path);
+GList               *ibus_observed_path_traverse            (IBusObservedPath   *path,
+                                                             gboolean            dir_only);
 
 /**
  * ibus_observed_path_check_modification:
